#### CHAPTER 2 (using RStudio)


# define directory and libraries
setwd(“SET YOUR OWN PATH“)

library(foreign) 	#to load stata dataset
library(lme4) 		#mlm functions
# library(arm) 		#to calculate se's on mlm
library(plyr) 		#to reshape data
library(lmerTest) 	#to include p value in the mixed model summary
library(lattice) 	#required for dotplot function

# load data - needs to be in Stata version 12 - doesn't work with 13
#  use Stata saveold command
ch2 <- read.dta("data_for_R_chapter_2.dta", convert.factors = FALSE)

# Define regions, as factors
ch2$region <- factor(ch2$region,
    levels = c("3601", "3602", "3603", "3604", "3605", "3606", "3607", "3608"),
    labels = c("ACT", "NSW", "VIC", "QLD", "SA", "WA", "TAS", "NT") )
# Tabulate the number of students in each region
table(ch2$region)

# Table 2.1
# Fit a simple linear model with reading score as dependent and parental status as independent variables.
ols2_1 <- lm(z_read ~ pos, data = ch2)
summary(ols2_1)

### grand mean center pos
# Center parental status by subtracting the mean parental status. na.rm = TRUE, removes missing values, 
# while calculating the mean. Otherwise R returns a missing value (NA)
ch2$cen_pos <- ch2$pos - mean(ch2$pos, na.rm = TRUE)

### OR grand mean center pos
# Another, more direct, way to center the vector or a matrix of values
# ch2$cen_pos2 <- scale(ch2$pos, scale = F)

# Table 2.2
# Fit a linear model using centered parental status as an independent variable.
ols2_2 <- lm(z_read ~ cen_pos, data = ch2)
summary(ols2_2)

# Figure 2.1
# Plot centered parental status along the x axis and reading scores along the y axis
plot(ch2$cen_pos, ch2$z_read)

# Abline function plots the regression line and can take manually specified coefficient and intercept, 
# or it can take it directly from the output of 'lm' function, or from the linear model object (ols2_2).
abline(a = 8.931e-11, b = 1.977e-02, col="blue")
abline(lm(z_read ~ cen_pos, data = ch2), col = "red")
abline(ols2_2, col = "green")

# Table 2.3 xtreg not replicated here

# Table 2.4
# Fit a null model with random intercepts. 
# 'lmer' function uses a particular syntax to specify the 'random' component of the model.
# It takes the following structure: 
# dependent variable ~ fixed effect variable + (random effect variable | grouping variable for random intercept)
# In the following formula fixed effect is substituted with 1, as we're fitting a null model, i.e. there is
# no fixed effect. Similarly, at this stage we only estimating random intercepts and hence we substitute 
# random effect with 1 as well. By default 'lmer' function selects estimates to optimise Restricted
#  Maximum Likelihood (REML), we set this option to false in order to optimise log-likelihood criterion.
ri2_4 <- lmer(z_read ~ 1 + (1 | region), data = ch2, REML = FALSE)

# In the following summary Random effects are separated from Fixed effects. Within the Random effects 
# summary Intercept Std.Dev corresponds to level 2 standard deviation and Residual Std.Dev 
# corresponds to student-level standard deviation within each group. These correspond to sigma_u and
# sigma_e parameters of 'xtreg' function and to sd(_cons) and sd(Residual) parameters of the mixed 
# function in Stata.
summary(ri2_4)

# Table 2.5 - random intercept values for each region
table2_5 <- data.frame(ranef(ri2_4)$region)
table2_5

# A way of getting actual values of the region intercepts (overall intersect + random intersect values)
coef(ri2_4)

# Figure 2.3 caterpillar plot using region. condVar option makes 'ranef' function to compute 
# variance-covariance matrices of the random effects.
dotplot(ranef(ri2_4, condVar = TRUE), strip = F, ylab = "rank", 
    scales = list(y = list(at = seq(1,8,1),
    labels = seq(1,8,1))))$region

## This produces a plot where the Regions are labelled by the name, not by the rank. 
dotplot(ranef(ri2_4, condVar = TRUE), strip = F)

# It's easy in R to compare two mixed effect models (e.g. containing one or two random factors) 
# using 'anova' function. To compare mixed and null model (no random effect) we will need to 
# fit the latter separately using 'lm' function.
ri2_4_ols <- lm(z_read ~ 1, data = ch2)

# We can then calculate log likelihood ratio.
LLR = - 2*logLik(ri2_4_ols, REML=F) + 2*logLik(ri2_4, REML=F)
LLR

# And test whether it is significant, assuming chi-square distribution
pchisq(LLR, df=1, lower.tail=F)

## Calculate the ICC
# 'VarCorr' function allows us to extract variance and standard deviation for the Random effects 
# parameters of the model (Intercept and Residual). 'as.data.frame' command coerces the output 
# of the 'VarCorr' to a data.frame for easy access. We can then calculate ICC from the Intercept 
# and Residual variances as described in the text.
var.mat_2_4 <- as.data.frame(VarCorr(ri2_4))
var.mat_2_4$vcov[1]/(var.mat_2_4$vcov[1] + var.mat_2_4$vcov[2]) 

# Table 2.6 (as Table 2.4)

# Table 2.7
# This requires a non-R-base package. 
# We suggest using 'table'
length(table(ch2$schoolid)) 	#Number of schools
range(table(ch2$schoolid)) 	#Min and max number of students per school
mean(table(ch2$schoolid)) 	#Average number of students per school


# Table 2.8 xtreg using schoolid group not replicated here

# Table 2.9 -  Fit a null model with schoolid as grouping variable
ri2_9 <- lmer(z_read ~ 1 + (1 | schoolid), data = ch2, REML = FALSE)
summary(ri2_9)

## getting the ICC
var.mat_2_9 <- as.data.frame(VarCorr(ri2_9))
var.mat_2_9$vcov[1]/(var.mat_2_9$vcov[1] + var.mat_2_9$vcov[2])

# Log likelihood Ratio test
# Fit the null linear model
ri2_9_ols <- lm(z_read ~ 1, data = ch2)

# Calculate log likelihood ratio.
LLR = - 2*logLik(ri2_9_ols, REML=F) + 2*logLik(ri2_9, REML=F)
LLR

# And test where it is significant, assuming chi-square distribution
pchisq(LLR, df=1, lower.tail=F)

# Figure 2.4 caterpillar plot using schoolid
dotplot(ranef(ri2_9, condVar = TRUE), strip = F, ylab = "u0rank", 
        scales = list(y = list(at = seq(1,356,88), 
        labels = seq(1,356,88))))$schoolid

# Table 2.10 - School-specific intercepts. We extract the random effects using 'ranef' function and
# order the schools starting with the lowest intercept (we can use 'decreasing = T' option in 'order' 
# function to display 10 highest intercepts). We use 'head' function with 'n=10' option to limit 
# display to the top 10 schools.
table2_10 <- data.frame(ranef(ri2_9)$schoolid)
head(table2_10[order(table2_10[, 1]), , drop=FALSE], n = 10)

# Table 2.11 - Fit mixed effects model with parental status as fixed and schoolid as random effects
# ri2_11 <- lmer(z_read ~ cen_pos + (1 | schoolid), data = ch2, REML = FALSE)
summary(ri2_11)

# Figure 2.6
# First we create a new dataset using only first schools with schoolid from 1 to 10.
newdt <- ch2[ ch2$schoolid %in% 1:10,]

# Then we calculate reading scores for each student based on the model ri2_11 and the independent
# variables in the model (cen_pos and schoolid) - we don't need to specify them, as long as they
# have the same names as were used to fit the model.
newdt$z_read <- predict(ri2_11, newdata = newdt)

# We plot predicted student scores vs parental occupation status.
plot(newdt$cen_pos, newdt$z_read, cex=0.1, pch=19)

# Finally, we plot a regression line for each schoolid. Here use the formula expression:
# 'z_read ~ cen_pos' to determine the slope and intercept for each line (this fits a simple OLS model)
# using z_read and cen_pos as dependent and independent variables for a single schoolid at a time.
for (i in 1:10){
    lines(z_read ~ cen_pos, data=newdt[ newdt$schoolid == i,])
}

# We can also add horizontal grid and a vertical line to mark cen_pos=0
abline(h=seq(-1, 1.5, 0.5), col="grey")
abline(v=0, col="red")

## getting the ICC
var.mat_2_11 <- as.data.frame(VarCorr(ri2_11))
var.mat_2_11$vcov[1]/(var.mat_2_11$vcov[1] + var.mat_2_11$vcov[2])

# Table 2.12 - Here we take first 10 schools (by schoolid) and then rank them 
# to produce the table from the text
table2_12 <- data.frame(ranef(ri2_11)$schoolid)
head(table2_12[order(table2_12[1:10, 1], decreasing = T), , drop=FALSE], n = 10)

# Table 2.13 - Fit mixed effects model adding gender to the fixed component of the model.
ri2_13 <- lmer(z_read ~ 1 + cen_pos + female + (1 | schoolid), 
      data = ch2, REML = FALSE)
summary(ri2_13)

## getting the ICC
var.mat_2_13 <- as.data.frame(VarCorr(ri2_13))
var.mat_2_13$vcov[1]/(var.mat_2_13$vcov[1] + var.mat_2_13$vcov[2])

# Centering school size using number of schools not number of students
# First, we need to get a list of schools with the respective size values
# Option 1 (most straightforward): We extract the matrix of school IDs and sizes and use 'unique'
# function to return unique combinations of school id and size. Note, we have to use both id and size.
# Otherwise, the schools of the same size will get bundled together.
cen.sch.size <- unique(ch2[, c("schoolid", "schsize")])

# Option 2: using 'ddply' or 'tapply' functions. Since we need to get a size value for each 
# school, we can use one of these functions to compute mean for each schoolid. Since all 
# instances of a given schoolid have the same schsize value - the function will return that value
cen.sch.size <- ddply(ch2, .(schoolid), function(data) {mean(data$schsize, na.rm = TRUE)})
cen.sch.size <- data.frame(schoolid=1:356, tapply(ch2$schsize, ch2$schoolid, 
	function(x) {mean(x, na.rm = TRUE)}))

# Calculate the mean school size base on the number of schools
mean(cen.sch.size[,2])

# Center school size by the mean schsize
ch2$cen_size <- ch2$schsize - mean(cen.sch.size[,2])

# Table 2.14 - Fit mixed effects model adding centred school to the fixed component of the model.
ri2_14 <- lmer(z_read ~ 1 + cen_pos + female + cen_size + (1 | schoolid), data = ch2, REML = FALSE)
summary(ri2_14)

## getting the ICC
var.mat_2_14 <- as.data.frame(VarCorr(ri2_14))
var.mat_2_14$vcov[1]/(var.mat_2_14$vcov[1] + var.mat_2_14$vcov[2])

## Center parental occupational status by school.
# First we calculate mean parental occupation status for each school using 'ddply' function
by.sch.pos <- ddply(ch2, .(schoolid), function(data) {mean(data$pos, na.rm = TRUE)})

# or using 'tapply' function as an R-base option
by.sch.pos <- data.frame(schoolid=1:356, posschmeans = tapply(ch2$pos, ch2$schoolid, 
	function(x) {mean(x, na.rm = TRUE)}))
names(by.sch.pos)[2] <- "posschmeans"

# Merge the data frame containing mean parental status by school with the original data frame using 'schoolid' as the common column
ch2 <- merge(ch2, by.sch.pos, by = "schoolid")

# Finally subtract school-based mean parental status from the respective pos values
ch2$grcen_pos <- ch2$pos - ch2$posschmeans
  
# Table 2.15 - Fit a mixed effects model replacing global-mean centered parental status with the school-mean centered one.
ri2_15 <- lmer(z_read ~ 1 + grcen_pos + female + cen_size + (1 | schoolid), 
	data = ch2, REML = FALSE)
summary(ri2_15)

## Center percent girls using number of schools not number of students
cen.pcgirls <- ddply(ch2, .(schoolid), function(data) {mean(data$pcgirls, na.rm = TRUE)})
names(cen.pcgirls)[2] <- "bypcgirls"

# R-base option
cen.pcgirls <- data.frame(schoolid=1:356, bypcgirls = tapply(ch2$pcgirls, ch2$schoolid, 
	function(x) {mean(x, na.rm = TRUE)}))
mean(cen.pcgirls$bypcgirls)
ch2$cen_girls <- ch2$pcgirls - mean(cen.pcgirls$bypcgirls)

# Table 2.16 - Fit a mixed effects model using gender and percent of girls in a school, as well as interaction between them
# as fixed effects and schoolid as random effect. In R the interaction term is specified with a colon, i.e. female:cen_girls.
# As a short-hand we can use '*' to list both individual terms and their interaction. 
# I.e. in the following formula 'female * cen_girls' is a short-hand for 'female + cen_girls + female:cen_girls'.
ri2_16 <- lmer(z_read ~ 1 + female * cen_girls + (1 | schoolid), data = ch2, REML = FALSE)
summary(ri2_16)

# Figure 2.7 graph of interaction from Table 2.16
## First, we create predicted values from fixed part of multilevel model
ch2$girls[!is.na(ch2$cen_girls)] <- fitted(ri2_16)

## We then create an empty plot of relationship between fitted reading scores and percent of girls
plot(girls ~ cen_girls, type = "n", ylim = c(-.5, .5), ylab = "predicted reading", data=ch2)

# And plot regression line for female students
abline(lm(ch2$girls[ch2$female == 1] ~ ch2$cen_girls[ch2$female == 1]), col = "darkblue")
# and male students
abline(lm(ch2$girls[ch2$female == 0] ~ ch2$cen_girls[ch2$female == 0]), col = "darkred")
legend(.3, .1, c("girls", "boys"), fill = c("darkblue", "darkred"))


# Center TSR on number of schools not number of students
# Since TSR is a school-level variable, the most straighforward way to get the list of TSR is to extract unique combinations of 
# schoolid and stratio variables
cen.ts.ratio <- unique(ch2[, c("schoolid", "stratio")])

# Another option is to calculate school-based mean TSR
cen.ts.ratio <- ddply(ch2, .(schoolid), function(data) {mean(data$stratio, na.rm = TRUE)})
names(cen.ts.ratio)[2] <- "byratio"

# The same using base R function
cen.ts.ratio <- data.frame(schoolid=1:356, byratio = tapply(ch2$stratio, ch2$schoolid, 
	function(x) {mean(x, na.rm = TRUE)}))
mean(cen.ts.ratio$byratio, na.rm = TRUE)
ch2$cen_ratio <- ch2$stratio - mean(cen.ts.ratio$byratio, na.rm = TRUE)
mean(ch2$cen_ratio, na.rm = TRUE)

# Table 2.17 - Fit a mixed effects model using parental status, teacher-student ratio and their interaction as fixed effects
# and schoolid as random effect. We also use 'subset' argument to limit the dataset to male students.
ri2_17 <- lmer(z_read ~ 1 + cen_pos * cen_ratio + (1 | schoolid), data = ch2, subset=female==0, REML = FALSE)
summary(ri2_17)

# Figure 2.7 - Plot the relationship between parental status and reading scored at different levels of teacher-student-ratio.
# First we need to divide schools into tertiles based on their TSR. We use 'cut2' function from 'Hmisc' package. We calculate
# tertiles at the school level rather than student level.
library("Hmisc")

# Extract centered TSR for each school
cen.ts.ratio <- unique(ch2[, c("schoolid", "cen_ratio")])

# Split schools into tertiles by specifying 3 groups within 'cut2' function. We also set 'levels.mean' argument to TRUE
# to automatically get mean TSR value for each tertile. 
cen.ts.ratio$tertiles <- cut2(cen.ts.ratio$cen_ratio, g=3, levels.mean = T)

# Merge tertile-based mean TSR with the original data frame. We only use columns 1 and 3 from cen.ts.ratio to avoid also merging
# centered TSR.
ch2 <- merge(ch2, cen.ts.ratio[, c(1, 3)], by = "schoolid")

## We have several options for plotting the interaction between parental status and TSR

# Option 1 - Using original data and fitted reading scores
# Subset the original dataset to include only males and remove observations without TSR value
ch2.boys <- ch2[ ch2$female == 0 & !is.na(ch2$cen_ratio),]

# Assign fitted values to the respective observations
ch2.boys$fitted_z_read <- fitted(ri2_17)
head(ch2.boys)

# Re-label tertile variable
levels(ch2.boys$tertiles) <- c("Low_TSR", "Med_TSR", "High_TSR")

# Generate empty plot of relationship between fitted reading scores and parental status
plot(fitted_z_read ~ cen_pos, type="n", data = ch2.boys)
# And plot regression line for each level of TSR
abline(lm(ch2.boys$fitted_z_read[ch2.boys$tertiles == "Low_TSR"] ~ ch2.boys$cen_pos[ch2.boys$tertiles == "Low_TSR"]), col="darkblue")
abline(lm(ch2.boys$fitted_z_read[ch2.boys$tertiles == "Med_TSR"] ~ ch2.boys$cen_pos[ch2.boys$tertiles == "Med_TSR"]), col="darkred")
abline(lm(ch2.boys$fitted_z_read[ch2.boys$tertiles == "High_TSR"] ~ ch2.boys$cen_pos[ch2.boys$tertiles == "High_TSR"]), col="darkgreen")
abline(h=seq(-2, 2, 0.5), col="grey")
legend(15, -.5, levels(ch2.boys$tertiles), lty = 1, col=c("darkblue", "darkred", "darkgreen"))

# Note that this plot looks very different from the one produced by the Stata. One plausable explanation is that in Stata we hold all
# the variables except TSR constant across the range of parental status scores. In this version of the plot in R, predicted reading
# scores are based on the relevant school id and actual TSR within each of the tertiles. We implement the Stata approach in Option 2.

# Option 2 (closer to the approach in text) - Generate a dummy dataset over specific ranges and predict reading scores.
# We use 'expand.grid' function to generate all possible combinations of the following variables:
newdata <- expand.grid(
    # Parental occupation status in the range from -20 to 20 with a step of 5 points
    cen_pos=seq(-20, 20, 5), 
    # 3 levels of TSR corresponding to the mean of each tertile
    cen_ratio = as.numeric(levels(cen.ts.ratio$tertiles)),
    # Since we use schoolid as a random effect in the model, we have to include it in the dummy dataset. But in order to produce 
    # a single estimate for each level of TSR we need to pick a single schoolid. One sensible option is to choose a schoolid with 
    # a median Intercept value. We can use 'ranef' function to extract school level intercepts and pick the median one. The seemingly
    # incomprehensible code  below does just that: it calculates the median Intercept and returns the name of the raw that matches 
    # that median. Finally, the name is converted into a number.
    schoolid = as.numeric(rownames(ranef(ri2_17)$schoolid)[which(ranef(ri2_17)$schoolid == median(ranef(ri2_17)$schoolid[,1]))]))
newdata

# Once we have the dummy dataset we can predict the reading score for each observation.
newdata$z_read <- predict(ri2_17, newdata)

# If we were not happy with using a schoolid with the median intercept value, we could adjust the predicted scores. For instance,
# if we wanted to set the intercept to the mean school-level intercept value, we can subtract the difference between the mean and median
# intercept from all predicted reading scores. Similarly, we could change it to the value of the global intercept.
newdata$z_read <- newdata$z_read - (median(ranef(ri2_17)$schoolid[,1]) - mean(ranef(ri2_17)$schoolid[,1]))

# Finally, we can plot the predicted reading scored against parental status and colour them by the TSR level
plot(newdata$z_read ~ newdata$cen_pos, pch=c(16, 18, 17)[as.factor(newdata$cen_ratio)], cex=1, 
     col=c("darkblue", "darkred", "darkgreen")[as.factor(newdata$cen_ratio)], ylim=c(-.63, .23), main="Tertiles Mean")
abline(h=seq(-0.7, 0.1, 0.1), col="grey")
for (i in names(table(newdata$cen_ratio))){
    points(z_read ~ cen_pos, data=newdata[ newdata$cen_ratio==i,], type="l")
}
legend(x = -20, y=0.2, legend = c("Low TSR", "Med TSR", "High TSR"), col=c("darkblue", "darkred", "darkgreen"), pch=c(16, 18, 17))

# Table 2.18 - We can compare the models using 'anova' command. In addition to the goodness of fit statistics (AIC, BIC, logLik) 
# it also calculates the reduction in deviance and computes the p value based on Chi-square distribution.
anova(ri2_9,ri2_11)
anova(ri2_13,ri2_11)
anova(ri2_14,ri2_13)

# Table 2.19 - Calculate R-squared value for the model
# To keep consistent with the main text we will re-name the models accordingly
null <- lmer(z_read ~ 1 + (1 | schoolid), data = ch2, REML = FALSE)
model1 <- lmer(z_read ~ female + (1 | schoolid), data = ch2, REML = FALSE)
model2 <- lmer(z_read ~ cen_pos + female + (1 | schoolid), data = ch2, REML = FALSE)
model3 <- lmer(z_read ~ cen_pos + female + cen_size + (1 | schoolid), data = ch2, REML = FALSE)

# We can extract between and within group variance for each model, as previously, using 'VarCorr' function.
a <- as.data.frame(VarCorr(null))$vcov[1] # Null model between variance
b <- as.data.frame(VarCorr(null))$vcov[2] # Null model within variance
c <- c(as.data.frame(VarCorr(model1))$vcov[1], as.data.frame(VarCorr(model2))$vcov[1], as.data.frame(VarCorr(model3))$vcov[1]) # Target model between variance
d <- c(as.data.frame(VarCorr(model1))$vcov[2], as.data.frame(VarCorr(model2))$vcov[2], as.data.frame(VarCorr(model3))$vcov[2]) # Target model within variance
names(c) <- names(d) <- c("Model1", "Model2", "Model3")
# And calculate overall R-squared...
(x <- ((a + b) - (c + d)) / (a + b))
# ... Level 1 R-squared...
(x1 <- (b - d) / d)
# ... and Level 2 R-squared
(x2 <- (a - c) / c)

# Diagnostics
model3 <- lmer(z_read ~ female + cen_pos + cen_size + (1 | schoolid), data = ch2, REML = FALSE)
summary(model3)
# Plot a Histogram and quantile normal plots of Level 1 residuals
hist(resid(model3))
qqnorm(resid(model3))
qqline(resid(model3))

# Plot a Histogram and quantile normal plots of Level 2 residuals
hist(ranef(model1)$schoolid[,1])
qqnorm(ranef(model1)$schoolid[,1])
qqline(ranef(model1)$schoolid[,1])


