#### CHAPTER 3 (using RStudio)


# define directory and libraries
setwd("/Users/davidpevalin/Desktop/book_R/")
library(foreign) #to load stata dataset
library(lme4) #mlm functions
# library(arm) #to calculate se's on mlm
library(plyr) #to reshape data
library(lmerTest) #to include p value in the mixed model summary


# load data - needs to be in Stata version 12 - doesn't work with 13
#  use Stata saveold command
ch3 <- read.dta("data_for_R_chapter_3.dta", convert.factors = FALSE)

## Note! cen_escs variable has two NA values, which has a potential to mess things up a bit later on. To avoid this, we remove the corresponding observations from the dataset.
ch3 <- ch3[!is.na(ch3$cen_escs),]
# Re-label regions and convert into the factor variable
ch3$region <- factor(ch3$region,
      levels = c("3601", "3602", "3603", "3604", "3605", "3606", "3607", "3608"),
      labels = c("ACT", "NSW", "VIC", "QLD", "SA", "WA", "TAS", "NT") )
table(ch3$region)
# Convert school ids into a factor
ch3$schoolid <- factor(ch3$schoolid)

# Figure 3.2
# Make an object to aggregate intercepts and coefficients from individual OLS models
ols <- as.data.frame(matrix(data = NA, nrow = length(levels(ch3$schoolid)), ncol = 2))
dimnames(ols) <- list(levels(ch3$schoolid), c("Intercept", "Coef"))
# Fit a linear model and collect Intercept and coefficient for each schoolid
for (i in levels(ch3$schoolid)){
  ols[i,] <- coef(lm(ch3$z_read ~ ch3$cen_pos, subset = ch3$schoolid == i))
}

# Calculate summary statistics for the interceps and coefficients. We use apply function to run the appropriate function on each column (margin=2)
cbind(Obs = apply(ols, 2, length),
      Mean = apply(ols, 2, mean),
      St.Dev = apply(ols, 2, sd),
      Min = apply(ols, 2, min),
      Max =apply(ols, 2, max))

# List the schools with the lowest and highest coefficients
head(ols[order(ols$Coef),])
tail(ols[order(ols$Coef),])

# Merge school-wise intercepts and coefficients into the main dataset according to "schoolid" variable
ols$schoolid <- rownames(ols)
ch3 <- merge(ch3, ols, "schoolid")

# Calculate predicted reading score
ch3$yhat <- ch3$Intercept + (ch3$Coef * ch3$cen_pos)

# Plot regression lines for the first 12 schools
plot(ch3$cen_pos[ as.numeric(ch3$schoolid) < 13], ch3$yhat[ as.numeric(ch3$schoolid) < 13], pch=19, cex=0.1)
for (i in 1:12){
  lines(ch3$cen_pos[ ch3$schoolid == i], ch3$yhat[ ch3$schoolid == i])
}
abline(v=0, h=0, lwd=2, col="maroon")

# Table 3.1
# Fit a mixed effects model only using random intercept for comparison purposes
ri_comp <- lmer(z_read ~ 1 + cen_pos + (1 | schoolid), data = ch3, REML = FALSE)
# Add random coefficient for cen_pos by including it in the random part of the formula
rc3_1 <- lmer(z_read ~ 1 + cen_pos + (1 + cen_pos | schoolid), data = ch3, REML = FALSE)
summary(rc3_1)
# Use anova command to compare the models
anova(ri_comp,rc3_1)

# Table 3.2
# Fit a model of reading scores over family capital using only random intercept
ri3_2 <- lmer(z_read ~ 1 + cen_escs + (1 | schoolid), data = ch3, REML = FALSE)
summary(ri3_2)

# Table 3.3
# Add random coefficient for cen_escs to the previous model and compare the models
rc3_3 <- lmer(z_read ~ 1 + cen_escs + (1 + cen_escs | schoolid), data = ch3, REML = FALSE)
summary(rc3_3)
anova(ri3_2,rc3_3)

# Figure 3.3
# Use ranef function to extract the random intercept and coefficient for each school and plot the scatter with each dot representing a school.
plot(ranef(rc3_3)$schoolid, pch=19, cex=0.5, col="dodgerblue4")
abline(h=seq(-0.2, 0.6, 0.2), col="lightgrey")
abline(v=0, h=0, col="maroon", lwd=2)

# Table 3.4
# To calculate absolute values of the random effects, first extract them from the model object, as before.
abs.rand.effects <- ranef(rc3_3)$schoolid
# Add sample-wide intercept and coefficient (using "fixef" function) to the appropriate random effects
abs.rand.effects[,1] <- abs.rand.effects[,1] + fixef(rc3_3)[1]
abs.rand.effects[,2] <- abs.rand.effects[,2] + fixef(rc3_3)[2]
# Or we could use apply funciton to do it in a single step
# t(apply(abs.rand.effects, 1, function(x) { x + fixef(rc3_3)}))
# Calculate summary statistics using apply command
cbind(Obs = apply(abs.rand.effects, 2, length),
      Mean = apply(abs.rand.effects, 2, mean),
      St.Dev = apply(abs.rand.effects, 2, sd),
      Min = apply(abs.rand.effects, 2, min),
      Max =apply(abs.rand.effects, 2, max))

# Figure 3.4
# Plot a scatter of absolute random coefficient values
plot(abs.rand.effects, pch=4, col="dodgerblue4", cex=0.7)
abline(h=seq(0, 1, 0.2), col="lightgrey")
abline(v=fixef(rc3_3)[1], h=fixef(rc3_3)[2], col="maroon", lwd=2)

# Figure 3.5
# Plot regression lines for each of the schools from random coefficients model and individual OLS models.
# First we get predicted values from the random coefficient model. 
# We can get these directly from the model:
ch3$yhat.rc <- fitted(rc3_3)
# Or by running a predict function on the original dataset using the model object:
ch3$yhat.rc <- predict(rc3_3, ch3)
# Set the plot area to have two plots side by side
par(mfrow=c(1, 2))
# Plot fitted values and regression lines for each of the schools
plot(ch3$cen_escs, ch3$yhat.rc, pch=16, cex=0.3, type="p", main="Random Coefficient\n Regression Lines", ylim=c(-6, 2))
abline(h=seq(-6, 2, 2), col="lightgrey", lwd=0.5)
for (i in ch3$schoolid){
  lines(ch3$cen_escs[ch3$schoolid == i], ch3$yhat.rc[ch3$schoolid == i], col="dodgerblue4", lwd=0.5)
}

# We also need to fit school-wise individual models, as we did earlier for cen_pos
# Make an object to aggregate intercepts and coefficients from individual OLS models
ols <- as.data.frame(matrix(data = NA, nrow = length(levels(ch3$schoolid)), ncol = 2))
dimnames(ols) <- list(levels(ch3$schoolid), c("Intercept_escs", "Coef_escs"))
# Fit a linear model and collect Intercept and coefficient for each schoolid
for (i in levels(ch3$schoolid)){
  ols[i,] <- coef(lm(ch3$z_read ~ ch3$cen_escs, subset = ch3$schoolid == i))
}
# Merge school-wise intercepts and coefficients into the main dataset
ols$schoolid <- rownames(ols)
ch3 <- merge(ch3, ols, "schoolid")
# Calculate predicted reading score
ch3$yhat_escs <- ch3$Intercept_escs + (ch3$Coef_escs * ch3$cen_escs)

# Plot the regression lines from the individual OLS models
plot(ch3$cen_escs, ch3$yhat_escs, pch=16, cex=0.3, type="p", main="OLS Regression Lines", ylim=c(-6, 2))
abline(h=seq(-6, 2, 2), col="lightgrey", lwd=0.5)
for (i in ch3$schoolid){
  lines(ch3$cen_escs[ch3$schoolid == i], ch3$yhat_escs[ch3$schoolid == i], col="dodgerblue4", lwd=0.5)
}
# Reset to one plot per view
par(mfrow=c(1,1))

# Figure 3.9
# In order to plot the Level 2 variance, we need to extract variance values for random intercepts and coefficients, as well their covariance value. The most direct way to do this is to use VarCorr function
var.cor.mat <- as.data.frame(VarCorr(rc3_3))
var.ic <- var.cor.mat$vcov[1]
var.coef <- var.cor.mat$vcov[2]
cov.ic.coef <- var.cor.mat$vcov[3]
# Plot a function of Level 2 variance over a range of cen_escs
curve(var.ic + 2 * cov.ic.coef * x + var.coef * x^2, -4, 2, xlab="cen_escs", ylab="Estimates of between-school variance", col="dodgerblue4", lwd=2, ylim=c(0, 0.8))
abline(h=seq(0, 1, 0.2), col="lightgrey", lwd=0.5)

# Table 3.5
# Fit a model only including Random Intercept for comparison
ri_comp3_5 <- lmer(z_read ~ 1 + female + (1 | schoolid), data = ch3, REML = FALSE)
# Fit a model with random intercept and random coefficient, by including the female variable in the random portion of the formula.
rc3_5 <- lmer(z_read ~ 1 + female + (1 + female | schoolid), data = ch3, REML = FALSE)
summary(rc3_5)
# Test if the inclusion of the random coefficients improves model's fit.
anova(ri_comp3_5,rc3_5)
# Extract variance-covariance matrix from the model
as.data.frame(VarCorr(rc3_5))

# Calculate summary statistics for the random effects
rand.effects <- ranef(rc3_5)$schoolid
cbind(Obs = apply(rand.effects, 2, length),
      Mean = apply(rand.effects, 2, mean),
      St.Dev = apply(rand.effects, 2, sd),
      Min = apply(rand.effects, 2, min),
      Max =apply(rand.effects, 2, max))

# Table 3.6
# To see if it's worth including two random coefficients, first fit a model only using cen_escs as a random coefficient and then compare it to the model containing both cen_escs and female as random coefficients
rc_comp3_6 <- lmer(z_read ~ 1 + cen_escs + female + (1 + cen_escs | schoolid), data = ch3, REML = FALSE)
rc3_6 <- lmer(z_read ~ 1 + cen_escs + female + (1 + cen_escs + female | schoolid), data = ch3, REML = FALSE)
anova(rc_comp3_6,rc3_6)
summary(rc3_6)
# Extract variances, covariances and correlations from the model
as.data.frame(VarCorr(rc3_6))

# Figure 3.10
# Plot the relationship between the random coefficients in the model
rand.effects <- ranef(rc3_6)$schoolid
plot(rand.effects$female, rand.effects$cen_escs, pch=16, cex=0.7, col="dodgerblue4", xlim=c(-0.25, 0.4))
abline(h=seq(-0.2, 0.6, 0.2), col="lightgrey", lwd=0.5)
abline(h=0, v=0, col="maroon", lwd=1.5)

# Table 3.7
# Fit a model using two independent variables, as fixed effects, but only one of them as a random effect.
rc3_7 <- lmer(z_read ~ 1 + cen_pos + female + (1 + female | schoolid), data = ch3, REML = FALSE)
summary(rc3_7)

# Table 3.8
# Adding level 2 variable - create new school size variable with a "per 100 students" unit.
ch3$cen_size100 <- ch3$cen_size/100
# Add school size as a fixed effect variable.
rc3_8 <- lmer(z_read ~ 1 + cen_pos + female + cen_size100 + (1 + female | schoolid), data = ch3, REML = FALSE)
summary(rc3_8)
# Test if inclusion of school size improves model's fit.
anova(rc3_7,rc3_8)

# Figure 3.11
# To test the assumptions of the model, extract and plot residuals:
par(mfrow=c(2,2))
hist(resid(rc3_6), main="Distribution of residuals", 100)
qqnorm(resid(rc3_6), main="Normal QQ plot of residuals", col="dodgerblue4", pch=16, cex=0.5)
qqline(resid(rc3_6), col="maroon", lwd=1.5)
# Do the same for random effect intercept and coefficients
rand.effects <- ranef(rc3_6)$schoolid
# Random intercept residuals
hist(rand.effects$"(Intercept)", 100, main="Distribution of Random \nIntercept  residuals")
qqnorm(rand.effects$"(Intercept)", main="Normal QQ plot of Random \nIntercept  residuals", col="dodgerblue4", pch=16, cex=0.5)
qqline(rand.effects$"(Intercept)", col="maroon", lwd=1.5)
# Family capital random coefficient residuals
hist(rand.effects$cen_escs, 100, main="Distribution of family \ncapital RC residuals")
qqnorm(rand.effects$cen_escs, main="Normal QQ plot of family \ncapital RC residuals", col="dodgerblue4", pch=16, cex=0.5)
qqline(rand.effects$cen_escs, col="maroon", lwd=1.5)
# Female random coefficient residuals
hist(rand.effects$female, 100, main="Distribution of female \nRC residuals")
qqnorm(rand.effects$female, main="Normal QQ plot of female \nRC residuals", col="dodgerblue4", pch=16, cex=0.5)
qqline(rand.effects$female, col="maroon", lwd=1.5)

# Table 3.10 three level null model
# First fit a null model only with Level 2 (schoolid) random intercept for comparison purposes
rc3_10_comp <- lmer(z_read ~ 1 + (1 | schoolid), data = ch3, REML = FALSE)
# Followed by addition of Level 3 (region) random intercept
rc3_10 <- lmer(z_read ~ 1 + (1 | region) + (1 | schoolid), data = ch3, REML = FALSE)
# OR
# rc3_10 <- lmer(z_read ~ 1 + (1 | region/schoolid), data = ch3, REML = FALSE)
# Test if addition of the region as a random intercept improves model's fit
anova(rc3_10_comp, rc3_10)
summary(rc3_10)
as.data.frame(VarCorr(rc3_10))

# List the number of schools per region
colSums(table(unique(ch3[,c("schoolid", "region")])))

# Table 3.11 three level RI model
# Add fixed effect variables and compare to the null random intercept model
rc3_11 <- lmer(z_read ~ 1 + cen_pos + female + cen_size100 + (1 | region) + (1 | schoolid), data = ch3, REML = FALSE)
# OR
# rc3_11 <- lmer(z_read ~ 1 + cen_pos + female + cen_size100 + (1 | region/schoolid), data = ch3, REML = FALSE)
summary(rc3_11)
anova(rc3_10,rc3_11)

# Table 3.12 three level RI model gender RC at L2
# Add random coefficient for female at the school level and check if it improves model's fit
rc3_12 <- lmer(z_read ~ 1 + cen_pos + female + cen_size100 + (1 | region) + (1 + female | schoolid), data = ch3, REML = FALSE)
summary(rc3_12)

anova(rc3_11,rc3_12)
as.data.frame(VarCorr(rc3_12))

# Table 3.13 three level RI model gender RC at L2 and size RC at L3
# Include random coefficient for female variable at the school levels and school size at the region level and check if it improves model's fit
rc3_13 <- lmer(z_read ~ 1 + cen_pos + female + cen_size100 + (1 + cen_size100 | region) + (1 + female | schoolid) , data = ch3, REML = FALSE)
summary(rc3_13)
anova(rc3_12,rc3_13)

# Table 3.14 Modelling a binomial dependent variable
# We first create a new variable 'hiread' with cases in top 10% of reading scores marked as 1.
ch3$hiread <- as.numeric(ch3$z_read >= quantile(ch3$z_read, 0.9))
table(ch3$hiread)
# One of the ways in R to model binomial mixed effects model is to use glmer function.
# First we fit a model using hiread as a dependent variable and cen_pos and female as fixed effects, as well as adding a random intercept at the school level
rc3_14_comp <- glmer(hiread ~ 1 + cen_pos + female + (1 | schoolid), data = ch3, family=binomial)
# Then we add female as a random coefficient at the school level
rc3_14 <- glmer(hiread ~ 1 + cen_pos + female + (1 + female | schoolid), data = ch3, family = binomial)
# Note, that the fixed effects in the summary are log odds and need to be exponentiated in order to make the comparable with Stata output in the book. Notably, female coefficient in substantially higher compared to the one produced by Stata (2.913876 vs 2.375917).
summary(rc3_14)
# Finally compare the models. 
as.data.frame(VarCorr(rc3_14))
anova(rc3_14_comp, rc3_14)
#  Note that in contrast to Stata output, variance for random coefficients for female variable is non-negligible (0.09 vs 0 in Stata) and LRT suggests that adding female as random coefficient significantly improves model's fit.

# Table 3.15
# Fit an unweighted model, as previously
rc3_15_unwht <- lmer(z_read ~ 1 + female + cen_pos + cen_size + (1 | schoolid), data=ch3, REML = FALSE)
summary(rc3_15_unwht)
as.data.frame(VarCorr(rc3_15_unwht))
# Add weights to the models
rc3_15_wht <- lmer(z_read ~ 1 + female + cen_pos + cen_size + (1 | schoolid), weights = w_fschwt, data=ch3, REML = FALSE)
summary(rc3_15_wht)
as.data.frame(VarCorr(rc3_15_wht))
# Compare the models. Note that according to this comparison unweighted model provides a significantly better fit
anova(rc3_15_unwht, rc3_15_wht)
anova(rc3_15_wht, rc3_15_unwht)
# Compare fixed and random coefficients between the unweighted and weighted models
rbind(cbind(Unweighted = fixef(rc3_15_unwht), Weighted = fixef(rc3_15_wht)),
      var.RI=c(as.data.frame(VarCorr(rc3_15_unwht))$vcov[1], as.data.frame(VarCorr(rc3_15_wht))$vcov[1]),
      var.Resid=c(as.data.frame(VarCorr(rc3_15_unwht))$vcov[2], as.data.frame(VarCorr(rc3_15_wht))$vcov[2]))  
# Note, the results of the weighted model in R are largely comparable to those produced by Stata. The only notable difference is the residuals variance, which is much larger in R output.