# Extracting data from the models
# Given the extensive library of R packages it is well possible that there is a package that would create publishable tables from the model objects. We invite the readers to explore 'stargazer' package that allows creation of LaTeX tables from model output. The code here will mosly focus on how to extract relevant data from the models and how to coerce them into a format of your liking. Here we will reproduce the table 4.1 from the book and save it in csv and tab delimited formats for further editing.

# define the working directory
# setwd("/Users/davidpevalin/Desktop/book_R/")
# Load the necessary libraries
library(foreign) #to load stata dataset
library(lme4) #mlm functions
library(lmerTest) #to include p value in the mixed model summary
# library(arm) #to calculate se's on mlm
# library(plyr) #to reshape data

# load data - needs to be in Stata version 12 - doesn't work with 13
#  use Stata saveold command -  we'll use the data from chapter 3
rm(list=ls())
ch4 <- read.dta("data_for_R_chapter_4.dta", convert.factors = FALSE)


# Re-label regions and convert into the factor variable
ch4$region <- factor(ch4$region,
      levels = c("3601", "3602", "3603", "3604", "3605", "3606", "3607", "3608"),
      labels = c("ACT", "NSW", "VIC", "QLD", "SA", "WA", "TAS", "NT") )
table(ch4$region)
# Convert school ids into a factor
ch4$schoolid <- factor(ch4$schoolid)
# Create new school size variable with a "per 100 students" unit.
ch4$schsize100 <- ch4$schsize/100


# Reproduce table 4.1. ----------------------------------------------------

# Fit the models
rc_0 <- lmer(z_read ~ 1 + (1 | schoolid), data=ch4, REML = FALSE)
rc_1 <- lmer(z_read ~ 1 + female + (1 | schoolid), data=ch4, REML = FALSE)
rc_2 <- lmer(z_read ~ 1 + female + pos + pcgirls + schsize100 + (1 | schoolid), data=ch4, REML = FALSE)
# fixef command will return a list of fixed effect coefficients for each of the models. Unfortunately, base version of R can't bind vectors of differing lengths into a table. We therefore use a workaround, where we index our smaller vectors by the length of the longest vector 'seq(fixef(rc_2))'. This replaces the later positions that don't exist in the shorter vectors with NA.
tble <- cbind(fixef(rc_0)[seq(fixef(rc_2))], fixef(rc_1)[seq(fixef(rc_2))], fixef(rc_2))
# Rename the rows with the coefficient names from the longest vector
rownames(tble) <- names(fixef(rc_2))
colnames(tble) <- ls(pattern = "rc.*")
# Round the values to 5 decimal places
tble <- round(tble, 5)
# Replace NA values with empty spaces
tble[is.na(tble)] <- ""
# Get p values from the model object summary and combine them as above
pvals <- cbind(coef(summary(rc_0))[,5][seq(fixef(rc_2))],
               coef(summary(rc_1))[,5][seq(fixef(rc_2))],
               coef(summary(rc_2))[,5])
# Replace numeric p values with starts
pvals[pvals > 0.05] <- ""
pvals[as.numeric(pvals) < 0.0011] <- "***"
pvals[as.numeric(pvals) < 0.01] <- "**"
pvals[as.numeric(pvals) < 0.05] <- "*"
# Replace NA values with empty spaces
pvals[is.na(pvals)] <- ""

tble <- matrix(paste(as.matrix(tble), as.matrix(pvals)), nrow=nrow(tble), dimnames=dimnames(tble))

# Get variances of random effects intercept and residuals using VarCorr command
ref_var <- cbind(as.data.frame(VarCorr(rc_0))[,4],
                as.data.frame(VarCorr(rc_1))[,4],
                as.data.frame(VarCorr(rc_2))[,4])
ref_var <- round(ref_var, 5)
rownames(ref_var) <- c("var(Intercept)", "var(Residual)")

n_obs = c(nobs(rc_0), nobs(rc_1), nobs(rc_2))

# Get the metrics of the model fit
model.fit <- cbind(llikAIC(rc_0)$AICtab, llikAIC(rc_1)$AICtab, llikAIC(rc_2)$AICtab)
model.fit <- round(model.fit, 2)
# Iterate through the models and calculate ICC for each model as in Chapter 2. 
ICCs <- numeric(length=0)
for (i in ls(pattern = "rc.*")){
  var.mat_2_4 <- as.data.frame(VarCorr(get(i)))
  ICCs[i] <- var.mat_2_4$vcov[1]/(var.mat_2_4$vcov[1] + var.mat_2_4$vcov[2]) 
}
ICCs <- round(ICCs, 5)
# Combine all the parameters into a single table
tble <- rbind(tble, ref_var, N = n_obs, model.fit, ICC = ICCs)

# You can rename columns or rows of the table using 'rownames' and 'colnames' commands and remove the undesired rows. Or, of course, you could easily edit it outside of R.
tble <- tble[!rownames(tble) %in% c("N", "BIC", "AIC", "deviance", "df.resid"),]
colnames(tble) <- c("Null", "Null + gender", "Null + gender + controls")
rownames(tble) <- c("Intercept",
                    "Female (ref: male)",
                    "Parental Occupational Status",
                    "Percentage Girls",
                    "School Size (/100 students)",
                    "Level 1 variance", 
                    "Level 2 variance",
                    "Log Likelihood",
                    "ICC")

# Add the p values legend
tble <- rbind(tble, "* p<0.05, ** p<0.01, *** p<0.001"[seq(ncol(tble))])

# Save the table as a csv file for further formatting. We can use 'na' parameter to over-write NAs with an empty space.
write.csv(tble, file="table_4.1.csv", na = "")
# And as a tab delimited file. We need to explicitly specify tab ("\t") as a separator. Otherwise R will use a space.
write.table(tble, file="table_4.1.txt", na = "", sep = "\t")


# Reproduce table 4.2 to include random coefficient. ----------------------

# Fit the models
rc_0 <- lmer(z_read ~ 1 + (1 | schoolid), data=ch4, REML = FALSE)
rc_1 <- lmer(z_read ~ 1 + female + (1 | schoolid), data=ch4, REML = FALSE)
rc_2 <- lmer(z_read ~ 1 + female + pos + escs + schsize100 + (1 | schoolid), data=ch4, REML = FALSE)
rc_3 <- lmer(z_read ~ 1 + female + pos + escs + schsize100 + (1 + escs | schoolid), data=ch4, REML = FALSE)

# Make a table of fixed effects. Since rc_2 and rc_3 have the same number of fixed effects, we just need to add the extra set of coefficients
tble <- cbind(fixef(rc_0)[seq(fixef(rc_2))], fixef(rc_1)[seq(fixef(rc_2))], fixef(rc_2), fixef(rc_3))
# Rename the rows with the coefficient names from the longest vector
rownames(tble) <- names(fixef(rc_2))
colnames(tble) <- ls(pattern = "rc.*")
# Round the values to 5 decimal places
tble <- round(tble, 5)
# Replace NA values with empty spaces
tble[is.na(tble)] <- ""
# Get p values from the model object summary and combine them as above
pvals <- cbind(coef(summary(rc_0))[,5][seq(fixef(rc_2))], 
               coef(summary(rc_1))[,5][seq(fixef(rc_2))], 
               coef(summary(rc_2))[,5],
               coef(summary(rc_3))[,5])
# Replace numeric p values with starts
pvals[pvals > 0.05] <- ""
pvals[as.numeric(pvals) < 0.0011] <- "***"
pvals[as.numeric(pvals) < 0.01] <- "**"
pvals[as.numeric(pvals) < 0.05] <- "*"
# Replace NA values with empty spaces
pvals[is.na(pvals)] <- ""

tble <- matrix(paste(as.matrix(tble), as.matrix(pvals)), nrow=nrow(tble), dimnames=dimnames(tble))

# Get variances of random effects intercept and residuals using VarCorr command. Now that we have two extra values for the rc_3, we need to use the same trick to fill out the missing values in the earier models with NA.
ref_var <- cbind(as.data.frame(VarCorr(rc_0))[,4][seq(nrow(as.data.frame(VarCorr(rc_3))))],
                 as.data.frame(VarCorr(rc_1))[,4][seq(nrow(as.data.frame(VarCorr(rc_3))))],
                 as.data.frame(VarCorr(rc_2))[,4][seq(nrow(as.data.frame(VarCorr(rc_3))))],
                 as.data.frame(VarCorr(rc_3))[c(1, 4, 2, 3),4])
ref_var <- round(ref_var, 5)
rownames(ref_var) <- c("var(Intercept)", "var(Residual)", "var(escs)", "corr(Intercept - escs)")

n_obs = c(nobs(rc_0), nobs(rc_1), nobs(rc_2), nobs(rc_3))

# Get the metrics of the model fit
model.fit <- cbind(llikAIC(rc_0)$AICtab, llikAIC(rc_1)$AICtab, llikAIC(rc_2)$AICtab, llikAIC(rc_3)$AICtab)
model.fit <- round(model.fit, 2)
# Iterate through the models and calculate ICC for each model as in Chapter 2. 
ICCs <- numeric(length=0)
for (i in ls(pattern = "rc.*")){
  var.mat_2_4 <- as.data.frame(VarCorr(get(i)))
  ICCs[i] <- var.mat_2_4$vcov[1]/(var.mat_2_4$vcov[1] + var.mat_2_4$vcov[var.mat_2_4$grp == "Residual"]) 
}
ICCs <- round(ICCs, 5)
# Combine all the parameters into a single table
tble <- rbind(tble, ref_var, N = n_obs, model.fit, ICC = ICCs)

# You can rename columns or rows of the table using 'rownames' and 'colnames' commands and remove the undesired rows. Or, of course, you could easily edit it outside of R.
tble <- tble[!rownames(tble) %in% c("BIC", "AIC", "deviance", "df.resid"),]
colnames(tble) <- c("Null", "Null + gender", "Null + gender + controls", " + random coefficient for school")
rownames(tble) <- c("Intercept",
                    "Female (ref: male)",
                    "Parental Occupational Status",
                    "ESCS",
                    "School Size (/100 students)",
                    "Level 1 variance", 
                    "Level 2 variance",
                    "Random slope ESCS",
                    "Correlation of random slope ECSC and intercept",
                    "N",
                    "Log Likelihood",
                    "ICC")

# Add the p values legend
tble <- rbind(tble, "* p<0.05, ** p<0.01, *** p<0.001"[seq(ncol(tble))])

# Save the table as a csv file for further formatting. We can use 'na' parameter to over-write NAs with an empty space.
write.csv(tble, file="table_4.2.csv", na = "")
# And as a tab delimited file. We need to explicitly specify tab ("\t") as a separator. Otherwise R will use a space.
write.table(tble, file="table_4.2.txt", na = "", sep = "\t")


# Graphical representation of the models ----------------------------------

# Graphing the interaction between gender and pos
rc_4.4 <- lmer(z_read ~ 1 + female * pos + escs + schsize100 + (1 | schoolid), data = ch4, REML = F)
summary(rc_4.4)
library("Hmisc")
ch4$quartpos <- as.factor(as.numeric(cut2(ch4$pos, g = 4)))
rc_4.5 <- lmer(z_read ~ 1 + female * quartpos + escs + schsize100 + (1 | schoolid), data = ch4, REML = F)
summary(rc_4.5)

# Create a new dataset to include all levels of female and pos quartiles, as well as mean values for schsize100 and escs. Do not include schoolid, as we want the predictions to use the common intercept.
new.data <- expand.grid(female=as.factor(0:1), quartpos = as.factor(1:4), escs = mean(ch4$escs, na.rm = T), schsize100 = mean(ch4$schsize100))

# Predict the z_read values based on the new data. re.form=NA tells the predict function not to use any particular levels of random effects in the prediction - i.e. use the common intercept.
new.data$z_read <- predict(rc_4.5, new.data, re.form=NA)

# predict function can calculate standard errors for linear models, but not for mixed models, as standard error caclulation for the latter is non-trivial. We use a work-around to calculate confidence intervals based on the variance of fixed effects ONLY from here: http://glmm.wikidot.com/faq . You can also include random effects variance in the calculation, but it doesn't seem to be what Stata is doing, so we don't include it here.

# Extract design matrix from the model. Note that it only includes fixed effects
mm <- model.matrix(terms(rc_4.5), new.data)

# We could also calculate the predicted values by matrix-multiplying the design matrix by the vector of fixed effect coefficients, but we use the predict function instead.
mm %*% fixef(rc_4.5) == new.data$z_read

# Calculate variances of the predictions, as a diagonal of the matrix product of XVX', where X is the design matrix and V is the variance-covariance matrix of the parameters extracted from the model object.
fixed.ef.var <- diag(mm %*% vcov(rc_4.5) %*% t(mm))

# Calculate 95% confidence interval from the variances based on the normal approximation
new.data$CI.lo <- new.data$z_read - 1.96 * sqrt(fixed.ef.var) 
new.data$CI.hi <- new.data$z_read + 1.96 * sqrt(fixed.ef.var)

# Plot z_read scores against pos quartiles separately for males and females
plot(as.numeric(new.data$quartpos), new.data$z_read, ylim = c(-0.4, 0.4), pch=16, cex=0.3, ylab="Linear Prediction, Fixed Portion", xlab = "4 quantiles of POS")
abline(h=seq(-.4, .4, .2), col="lightgrey", lwd=0.5)
for (i in 0:1){
  # Plot the line for the predictions
  lines(as.numeric(new.data$quartpos)[new.data$female == i], new.data$z_read[new.data$female == i], col=c("darkblue", "darkred")[i+1], lwd=2)
  # Plot upper CI
  lines(as.numeric(new.data$quartpos)[new.data$female == i], new.data$CI.lo[new.data$female == i], col=c("skyblue3", "salmon")[i+1], lwd=2, lty=2)
  # Plot lower CI
  lines(as.numeric(new.data$quartpos)[new.data$female == i], new.data$CI.hi[new.data$female == i], col=c("skyblue3", "salmon")[i+1], lwd=2, lty=2)
}
legend("topleft", legend = c("female=0", "female=1"), fill=c("darkblue", "darkred"), bty = "n")

# Graphing the interaction between gender and schoolsize
rc_4.6 <- lmer(z_read ~ 1 + female * schsize100 + escs + pos + (1 | schoolid), data = ch4, REML = F)
summary(rc_4.6)

# Create a new dataset
new.data <- expand.grid(female=as.factor(0:1), schsize100 = seq(1, 26, 5), escs = mean(ch4$escs, na.rm = T), pos = mean(ch4$pos))

# Predict the z_read values based on the new data ignoring the different levels of random effects.
new.data$z_read <- predict(rc_4.6, new.data, re.form=NA)

# Calculate 95% confidence intervals for the predictions
mm <- model.matrix(terms(rc_4.6), new.data)
fixed.ef.var <- diag(mm %*% vcov(rc_4.6) %*% t(mm))

# Calculate 95% confidence interval from the variances based on the normal approximation
new.data$CI.lo <- new.data$z_read - 1.96 * sqrt(fixed.ef.var) 
new.data$CI.hi <- new.data$z_read + 1.96 * sqrt(fixed.ef.var)

# Plot z_read scores against pos quartiles separately for males and females - Confidence intervals, as error bars
plot(as.numeric(new.data$schsize100), new.data$z_read, ylim = c(-0.7, 1), pch=16, cex=1, ylab="Linear Prediction, Fixed Portion", xlab = "School Size/100 students", col=c("darkblue", "darkred")[new.data$female])
abline(h=seq(-.5, 1, .5), col="lightgrey", lwd=0.5)
# Plot CI, as error bars
segments(new.data$schsize100, new.data$CI.lo, new.data$schsize100, new.data$CI.hi, col=c("darkblue", "darkred")[new.data$female], lwd=1.5)
segments(new.data$schsize100-0.2, new.data$CI.lo, new.data$schsize100+0.2, new.data$CI.lo, col=c("darkblue", "darkred")[new.data$female], lwd=1.5)
segments(new.data$schsize100-0.2, new.data$CI.hi, new.data$schsize100+0.2, new.data$CI.hi, col=c("darkblue", "darkred")[new.data$female], lwd=1.5)
# Plot the lines for the predictions
for (i in 0:1){
  lines(as.numeric(new.data$schsize100)[new.data$female == i], new.data$z_read[new.data$female == i], col=c("darkblue", "darkred")[i+1], lwd=2)
}
legend("topleft", legend = c("female=0", "female=1"), fill=c("darkblue", "darkred"), bty = "n")

# Plot z_read scores against pos quartiles separately for males and females - Confidence intervals, as area.
plot(as.numeric(new.data$schsize100), new.data$z_read, ylim = c(-0.7, 1), pch=16, cex=1, ylab="Linear Prediction, Fixed Portion", xlab = "School Size/100 students", col=c("darkblue", "darkred")[new.data$female])
abline(h=seq(-.5, 1, .5), col="lightgrey", lwd=0.5)
for (i in 0:1){
  # Plot the line for the predictions
  lines(as.numeric(new.data$schsize100)[new.data$female == i], new.data$z_read[new.data$female == i], col=c("darkblue", "darkred")[i+1], lwd=2)
  # Plot upper CI
  lines(as.numeric(new.data$schsize100)[new.data$female == i], new.data$CI.lo[new.data$female == i], col=c("skyblue3", "salmon")[i+1], lwd=2, lty=2)
  # Plot lower CI
  lines(as.numeric(new.data$schsize100)[new.data$female == i], new.data$CI.hi[new.data$female == i], col=c("skyblue3", "salmon")[i+1], lwd=2, lty=2)
}
legend("topleft", legend = c("female=0", "female=1"), fill=c("darkblue", "darkred"), bty = "n")


# Reproduce Table 3 from the manuscript -----------------------------------

# Using the same model as above.
rc_4.6 <- lmer(z_read ~ 1 + female * schsize100 + escs + pos + (1 | schoolid), data = ch4, REML = F)
summary(rc_4.6)
# Extract fixed effects and standard errors from the model. Standard errors are extracted as in previous examples by taking a square root of the diagonal values of the variance-covariance matrix.
fixed.effects <- data.frame(B = fixef(rc_4.6), SE = sqrt(diag(vcov(rc_4.6))))
fixed.effects <- round(fixed.effects, 3)
# Get p values from the model object summary and combine them with the fixed effects
pvals <- coef(summary(rc_4.6))[,5]
# Replace numeric p values with starts
pvals[pvals > 0.1] <- ""
pvals[as.numeric(pvals) < 0.0011] <- "***"
pvals[as.numeric(pvals) < 0.01] <- "**"
pvals[as.numeric(pvals) < 0.05] <- "*"
pvals[as.numeric(pvals) < 0.1] <- "+"
# Replace NA values with empty spaces
pvals[is.na(pvals)] <- ""
fixed.effects$B <- paste(fixed.effects$B, pvals, sep="")
fixed.effects$SE <- paste("(", fixed.effects$SE, ")", sep="")

# Extract variance components for the random effects
var.mat_4.6 <- as.data.frame(VarCorr(rc_4.6))
rownames(var.mat_4.6) <- var.mat_4.6$grp
reff.var <- round(var.mat_4.6[,4, drop=F], 3)

# Calculate ICC and log Likelihood
ICC <- round(var.mat_4.6$vcov[1]/(var.mat_4.6$vcov[1] + var.mat_4.6$vcov[2]),3)
log.Lik <- as.numeric(logLik(rc_4.6))

# Combine everything into a single table
# First fill out the part of the table with a single column with NA
reff.and.model.fit <- cbind(rbind(reff.var, ICC = ICC, LogLikelihood = log.Lik), NA)
# Combine the tables as matrices to avoid the problem of non-consistent column names
tble <- rbind(as.matrix(fixed.effects), as.matrix(reff.and.model.fit))

# Rename the rows
rownames(tble) <- c("Intercept",
                    "Female (ref: male)",
                    "School Size (/100 students)",
                    "Family Capital",
                    "Parental Occupational Status",
                    "Female * School Size",
                    "Level 1 variance", 
                    "Level 2 variance",
                    "ICC",
                    "Log Likelihood")

# Add the table legend
tble <- rbind(tble, "Standard Errors in parentheses"[seq(ncol(tble))])
tble <- rbind(tble, "+ p<0.1, * p<0.05, ** p<0.01, *** p<0.001"[seq(ncol(tble))])

# Save the table as a csv or a tab-delimited file
write.csv(tble, file="table_4.3.csv", na = "")
write.table(tble, file="table_4.3.txt", na = "", sep = "\t")
